// $Id: ConstAddrRndReqMaker.cc,v 1.22.2.2 1998/09/22 07:43:24 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "IOBuffer.h"
#include "ConstAddrRndReqMaker.h"
#include "XactLog.h"
#include "Gadgets.h"
#include "Rnd.h"
#include "globals.h"

#include <math.h>

void ConstAddrRndReqMaker::genObjId() {
	if (TheOpt.theRndSeed > 0) {
		// randomly distributed object ids
		static RndGen theRnd(TheOpt.theRndSeed);
		// XXX: hard coded zipf(1) distribution
		theCont.theReqHdr->objId((int)
			(0.5 + exp(TheWorld.zipfOmega()*theRnd() - TheEMConst)));
	} else {
		// unique ids (e.g. for populating the cache)
		static int uniqueueId = TheOpt.theSeqStart;
		theCont.theReqHdr->objId(uniqueueId++);
		return;
	}
}

void ConstAddrRndReqMaker::doContent() {
	static int unique_id = TheOpt.theUniqStart;
	IOBuffer &buf = theConn->theWriterBuf;

	genObjId();
	if (theRec)
		TheXactLog.setId(theRec);

	// form HTTP request
	// req-line prefix
	buf << "GET http://" << theAddr.host() << ':' << theAddr.port() << '/';

	// url
	buf << '_' << theCont.theReqHdr->objId() << '/' << TheWorld.id();

	// signature (XXX: hardcoded!)
	buf << "/polyGraph";

	if (unique_id >= 0) buf << "/" << unique_id++;

	// req-line trailer
	//buf << " HTTP/1.1" << crlf; // note: 1.1 Implies persistency
	buf << " HTTP/1.0" << crlf;

	// headers
	if (theRec)
		buf << "X-Xact: " << theRec->id() << crlf;
	buf << "Accept: */*" << crlf;
	// ...

	// add crlf
	buf << crlf;

	// done!
	maxOffsetHere();

	// XXX: kludge, should form a real header, but that would lead to copying/allocating stuff...
	theCont.theReqHdr->hdrSize(sizeTotal());
	sendContChange();
}
