// $Id: RndRepMaker.cc,v 1.25.2.1 1998/09/22 07:43:40 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "IOBuffer.h"
#include "RndRepMaker.h"
#include "Rnd.h"
#include "XactLog.h"
#include "Log.h"
#include "Gadgets.h"
#include "globals.h"


Size RndRepMaker::calcRepSize() const {
	// XXX: hard coded exponential distribution
	static ExpDistr TheSizeDistr(new RndGen(), TheWorld.objSizeMean());

	const int objId = theCont.theReqHdr->objId();
	Assert(objId > 0 && objId <= TheWorld.capacity());
	// seed random generator with objId
	TheSizeDistr.rndGen().seed((unsigned)objId);
	Size sz = (size_t) TheSizeDistr();
	// XXX: hard coded size limits:
	// most popular objects should not be too big (100*id + 10KB)
	// objects should not exceed 50MB
	sz %= min((Size)(100*objId + Size::KB(10)), Size::MB(50));
	// the object must be at least CBlock size
	return max(sz, (Size)32);
}

void RndRepMaker::make200Prefix() {

	// determine the size of the reply
	theCBlock.contSize(calcRepSize());
	
	// put HTTP reply headers
	IOBuffer &buf = theConn->theWriterBuf;
	buf	<< "HTTP/1.0 200" << crlf; // 1.1
	if (theRec && theRec->id() > 0)
		buf << "X-Xact: " << theRec->id() << crlf;
	buf	<< "Content-Length: " << (int)theCBlock.contSize() << crlf
		<< crlf;

	maxOffset(workOffset() + theCBlock.contSize());
	Notepad << this << " 200 rep; cont size:" << theCBlock.contSize() << " total: " << sizeTotal() << " buf: " << buf.contSize() << ende;

	// put CBlock
	buf << theCBlock;
}

void RndRepMaker::make304Prefix() {

	theCBlock.contSize(0);
	
	// put HTTP reply headers
	IOBuffer &buf = theConn->theWriterBuf;
	buf	<< "HTTP/1.0 304" << crlf // 1.1
		<< "Cache-Control: public,max-age=86400" << crlf // one day
		<< crlf;

	maxOffsetHere();
	Notepad << this << " 304 rep; size total: " << sizeTotal() << " buf: " << buf.contSize() << ende;
}

// append random content to the io buffer
void RndRepMaker::doContent() {
	Assert(theOtherContState == csValid);

	IOBuffer &buf = theConn->theWriterBuf;

	if (theCBlock.contSize() < 0)
		theCont.theReqHdr->ims() ? make304Prefix() : make200Prefix();

	const Size size_app(min(buf.bufSize(), sizeLeft()));
	if (size_app > 0)
		buf.appended(size_app); // just use buffer area, no content

	// check if we want to close the connection
	if (thePersFlag && sizeLeft() <= 0)
		buf.eod(true);
}
