#ifndef _EVENT_CLASS_H_
#define _EVENT_CLASS_H_ "$Id: Event.h,v 1.5.2.1 1998/09/22 07:44:05 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"
#include "Array.h"

class ostream;

// an event is a piece of code executed by the EventQueue
class Event {
	public:
		virtual ~Event() {}
		virtual void cancel() {}
		virtual void execute() = 0;

		virtual ostream &print(ostream &os) const { return os; }
};

inline ostream &operator <<(ostream &os, const Event &ev) { return ev.print(os); }


// manager of events
// (not a child of Queue because it is not really a queue but more like
// a buffer or accumulator)
class EventQueue {
	public:
		EventQueue(): theExecCount(0) {}
		virtual ~EventQueue();

		// int execCount() const { return theExecCount; }
		bool empty() const { return !theEvents.count(); }

		void enqueue(Event *ev) { theEvents.append(ev); noteIn(ev); }
		int step(); // returns the number of events executed

	protected:
		// debugging hooks
		void noteIn(const Event *ev) const;
		void noteOut(const Event *ev) const;

	protected:
		Array<Event*> theEvents;
		int theExecCount;
};

extern EventQueue TheEventQueue;

#endif
