// $Id: Identity.h,v 1.1.2.1 1998/09/22 07:44:09 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _IDENTITY_H_
#define _IDENTITY_H_

#include <iostream.h>

// Identity is a [virtual] base class for classes that want to have an id
// note: if two classes implement their own Ids, then their common child
//       would have two identities which is unnatural; also having one
//       base class makes it easy to support hierarchical Ids like
//       <parent_id:child_id> or (first_name, middle_name)

class Identity {
	public:
		Identity(): theId(++TheLastId) {}
		virtual ~Identity() {}

		int id() const { return theId; }
		virtual const char *name() const { return "anonymous"; }

		virtual ostream &printId(ostream &os) const { return os << theId; }

	protected:
		int theId;

	private:
		static int TheLastId;
};

// print Id instead of a pointer (pointers change due to inheritance!)
inline
ostream &operator <<(ostream &os, const Identity *i) { return i ? i->printId(os) : (os << "00"); }

#endif
