#ifndef _PAUSE_H_
#define _PAUSE_H_ "$Id: Pause.h,v 1.3.2.1 1998/09/22 07:44:15 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Time.h"
#include "Rnd.h"

// returnes a positive delay if an activity should pause for that time
// returnes a zero or negative delay if an activity should continue right away

class Pause {
	public:
		virtual ~Pause() {}

		virtual Pause *clone() const { return new Pause(); }

		virtual Time pause() { return Time(0,0); }
};

class ConstPause: public Pause {
	public:
		ConstPause(Time aDelay): theDelay(aDelay) {}

		virtual Pause *clone() const { return new ConstPause(theDelay); }
		virtual Time pause() { return theDelay; }
	
	protected:
		Time theDelay;
};

class RndPause: public Pause {
	public:
		RndPause(RndDistr *aDistr): theDistr(aDistr) {}
		~RndPause() { delete theDistr; }

		virtual Pause *clone() const { return new RndPause(theDistr->clone()); }
		virtual Time pause() { return Time::Secd(theDistr->trial()); }

	protected:
		RndDistr *theDistr;
};

#endif
