#ifndef _SELECT_LOOP_H_
#define _SELECT_LOOP_H_ "$Id: SelectLoop.h,v 1.12.2.1 1998/09/22 07:44:25 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h" /* to reset FD_SETSIZE, in part */

#include "Time.h"
#include "SafePtr.h"
#include "Identity.h"

#include <sys/types.h>
#include <unistd.h>

typedef enum IODir { ioNone = -1, ioRead = 0, ioWrite = 1, ioExcept = 2 };

// an object capable of doing FD-based IOs (XXX: ugly name)
class IOer: public virtual Identity, public virtual SelfAcc {
	public:
		typedef void (IOer::*IOMethod)(int fd);

	public:
		IOer() {}
		virtual ~IOer() {}

		virtual void readReady(int fd) { cannotHandle(fd); }
		virtual void writeReady(int fd) { cannotHandle(fd); }
		virtual void exceptReady(int fd) { cannotHandle(fd); }

	protected:
		virtual void cannotHandle(int) { ::abort(); }
};

// reservation of the select loop entry
class SelectReserv {
	public:
		SelectReserv(): theFD(-1), theDir(ioNone) {}
		SelectReserv(int aFD, IODir aDir): theFD(aFD), theDir(aDir) {}

		operator bool() const { return reserved(); }
		bool reserved() const { return fd() >= 0 && theDir != ioNone; }

		int fd() const { return theFD; }
		IODir dir() const { return theDir; }

		void clear() { theFD = -1; }

	protected:
		int theFD;
		IODir theDir;
};

// wrapped arrounf fd_set
class FD_Set {
	public:
		FD_Set();
		~FD_Set();

		int maxFD() const { return theMaxFD; }
		int resCount() const { return theResCount; }
		bool active() const { return theMaxFD >= 0; }
		bool readyActive() const { return theReadyMaxFD >= 0; }
		fd_set *makeReadySet() { theReadySet = theSet; theReadyMaxFD = theMaxFD; return readyActive() ? &theReadySet : 0; }

		void setFD(int fd, IOer *p);

		int doIOs(IOer::IOMethod method, const char *methodName);

		bool isSet(int fd) const { return FD_ISSET(fd, &theSet); }
		IOer *operator [](int fd) { return fd <= theMaxFD ? theIOers[fd] : 0; }

	protected:
		IOer **theIOers;
		int theMaxFD;
		int theReadyMaxFD;  // used in select loop only
		int theResCount;

		fd_set theSet;
		fd_set theReadySet; // used in select loop only
};

// wrapper around select(2) system call
class SelectLoop {
	public:
		SelectLoop();
		~SelectLoop() {}

		bool isIdle() const { return theMaxFD < 0; }

		int step(Time timeout); // returns #ready fds and -1 on error

		SelectReserv setFD(int fd, IODir dir, IOer *p);
		void clearFD(SelectReserv &res) { clearFD(res.fd(), res.dir()); res.clear(); }
		void clearFD(int fd, IODir dir);

	protected:
		void syncMaxFD();

	protected:
		FD_Set theSets[3]; // read/write/exception
		int theMaxFD;
		int theResCount;   // #clients == #reserved fds
};

extern SelectLoop TheSelectLoop;

#endif
