#ifndef _TIME_CLASS_H_
#define _TIME_CLASS_H_ "$Id: Time.h,v 1.11.2.1 1998/09/22 07:44:31 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"

#include <assert.h>
#include <sys/time.h>

class istream;
class ostream;

// a wrapper arround "struct timeval"
class Time: public timeval {
	public:
		static Time Now();
		static Time Sec(long sec) { return Time(sec, 0); }
		static Time Secd(double dsec);

	public:
		Time() { tv_sec = tv_usec = -1; }
		Time(long sec, long usec) { tv_sec = sec; tv_usec = usec; }
		
		bool operator ==(const Time &tm) const { return tv_sec == tm.tv_sec && tv_usec == tm.tv_usec; }
		bool operator  >(const Time &tm) const { return tv_sec > tm.tv_sec || (tv_sec == tm.tv_sec && tv_usec > tm.tv_usec); }
		bool operator >=(const Time &tm) const { return tv_sec > tm.tv_sec || (tv_sec == tm.tv_sec && tv_usec >= tm.tv_usec); }
		bool operator  <(const Time &tm) const { return tv_sec < tm.tv_sec || (tv_sec == tm.tv_sec && tv_usec < tm.tv_usec); }
		bool operator <=(const Time &tm) const { return tv_sec < tm.tv_sec || (tv_sec == tm.tv_sec && tv_usec <= tm.tv_usec); }

		bool operator ==(int t) const { assert(!t); return !tv_sec && !tv_usec; }
		bool operator  >(int t) const { assert(!t); return tv_sec > 0 || (tv_sec == 0 && tv_usec  > 0); }
		bool operator >=(int t) const { assert(!t); return tv_sec > 0 || (tv_sec == 0 && tv_usec >= 0); }
		bool operator  <(int t) const { assert(!t); return tv_sec < 0 || (tv_sec == 0 && tv_usec  < 0); }
		bool operator <=(int t) const { assert(!t); return tv_sec < 0 || (tv_sec == 0 && tv_usec <= 0); }

		Time &operator +=(const Time &tm);
		Time &operator -=(const Time &tm);

		long sec() const { return tv_sec; }
		long usec() const { return tv_sec*(long)1e6 + tv_usec; }
		double msecd() const { return tv_sec*1e3 + tv_usec/1e3; }
		double secd() const { return tv_sec*1e0 + tv_usec/1e6; }

		istream &load(istream &is);
		const char *load(const char *buf);
		ostream &print(ostream &os) const;

	protected:
		ostream &printInterval(ostream &os) const;
};

inline
Time operator +(Time t1, Time t2) {	return t1 += t2; }

inline
Time operator -(Time t1, Time t2) {	return t1 -= t2; }

inline
ostream &operator <<(ostream &os, const Time &tm) { return tm.print(os); }

inline
istream &operator >>(istream &is, Time &tm) { return tm.load(is); }


#endif
