// $Id: UseAcc.h,v 1.4.2.1 1998/09/22 07:44:33 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _USE_ACC_H_
#define _USE_ACC_H_

// this is module used to create SafePtr

#include "AppDef.h"

// accounting info on a object usage
class UseRec {
	public:
		UseRec(): theUseCount(0), isGone(false) {}
		~UseRec() { Assert(abandoned()); }

		bool used() const { return theUseCount > 0; }
		bool abandoned() const { return theUseCount == 0; }
		bool gone() const { return isGone; }
		bool valid() const { return !gone() && used(); }

		int useCount() const { return theUseCount; }

		void use() { theUseCount++; }
		bool abandon() { return --theUseCount == 0; }
		void beGone() { isGone = true; }

	protected:
		short unsigned int theUseCount;
		bool isGone;
};

// accountant for UseRec
// automagically counts the number of users and destroys when abandoned()
class UseAcc {
	public:
		UseAcc(UseRec *aUseRec) { set(aUseRec); }
		~UseAcc() { unset(); }

		UseRec *useRec() const { return theUseRec; }

	protected:
		void set(UseRec *aUseRec) { if ((theUseRec = aUseRec)) theUseRec->use(); }
		void unset() { if (theUseRec && theUseRec->abandon()) delete theUseRec; }

	protected:
		mutable UseRec *theUseRec;
};

// an object to put into the "creator" object to control 
// "isGone" state and creation of the UseRec
class SelfAcc: public UseAcc {
	public:
		SelfAcc(): UseAcc(new UseRec) {}
		virtual ~SelfAcc() { beGone(); }

		bool gone() const { return theUseRec->gone(); }
		void beGone() { theUseRec->beGone(); }

	private: /* disable these */
		SelfAcc(const SelfAcc &): UseAcc(0) { abort(); }
		void operator =(const SelfAcc &) { abort(); }
};

// an object to put into the "user" object
// does not affect "isGone" state and creation of UseRec.
// but can be assigned and copied
class UserAcc: public UseAcc {
	public:
		UserAcc(): UseAcc(0) {}
		UserAcc(const UserAcc &ua): UseAcc(ua.theUseRec) {}

		UserAcc &operator =(const UserAcc &ua) { reset(ua.useRec()); return *this; }
		//UserAcc &operator =(const UserAcc &ua) { return *this = ua.useRec(); }
		//UserAcc &operator =(UseRec *ur) { reset(ur); return *this; }

	protected:
		void reset(UseRec *ur) { unset(); set(ur); }
		bool safe() const { return theUseRec && theUseRec->valid(); }
};

#endif
