#!/usr/bin/perl -w
use strict;

#
# cuts out a given column from a table with a heading
#

require FileHandle;

my ($col_name, $fname) = @ARGV;
$fname ||= './shotr /tmp/t.shot|';
die(&usage()) unless $col_name && $fname;


exit(&main() == 0);

sub main {
	my $fh = new FileHandle($fname);
	die("cannot open $fname, $!; stopped") unless $fh;

	# read until data line is found
	my ($heading, $first_data) = ('', '');
	while (<$fh>) {
		chomp();
		$first_data = $_;
		last if /^\s*\d+/;
		$heading = $_;
	}

	my @heading = split(/\#?\s*\t\s*/, $heading);
	die("unknown file format (missing heading?), stopped") unless scalar @heading;

	my $col_id = -1;
	foreach (@heading, '') {
		$col_id++;
		last if /^$col_name$/i;
	}
	die("cannot find column $col_name in $heading, stopped") unless $col_id >= 0 && $col_id <= $#heading;

	# print output header
	printf("# Generated from '%s' by %s\n", $fname, join(' ', $0, @ARGV));
	printf("# column %s (%d)\n", $col_name, $col_id);

	# print the column (and the first column!)
	my $time = 0;
	my $i = 0;
	while (<$fh>) {
		chomp();
		my @data = split(/\s*\t\s*/, $_);
		my $col = $data[$col_id] - $data[$col_id+2];
		die("unknown line format near '$_', stopped") unless defined $data[0] && defined $col;
		printf("%5d %11.6f %9d\n", $i, $time/1e6, $col) if $col;
		$time += $data[0];
		$i++;
	}

	undef($fh);
	return 1;
}


sub usage {
	return << "USAGE";
usage: $0 <column_name> [file_name]
USAGE
}
