#!/usr/bin/perl -w
use strict;

# $Id: xjoin.pl,v 1.1 1998/08/22 20:43:09 rousskov Exp $

#
# Joins two transaction logs together
# performing necessary time adjustments.
#

# set tab-width 4

require FileHandle;

die("usage: $0 <client_log> <server_log>\n") unless @ARGV == 2;
my ($clFile, $svFile) = @ARGV;

# globals
my ($clFH, $svFH);
my ($clHdr, $svHdr) = ({ name => $clFile}, { name => $svFile });

my $JoinCount = 0;

exit(&main() == 0);

sub main {

	warn("joining $clFile with $svFile\n");

	$clFH = &openLog($clFile);
	$svFH = &openLog($svFile);

	&readHdr($clFH, $clHdr);
	&readHdr($svFH, $svHdr);

	&loop();

	undef $clFH;
	undef $svFH;

	return 1;
}

sub loop {
	my ($clLine, $svLine);
	while ($clFH && $svFH) {
		$clLine ||= $clFH->getline();
		$svLine ||= $svFH->getline();
		last if ! defined $clLine || ! defined $svLine;
		# print(" cl: $clLine sv: $svLine\n");
		my ($clId, $svId) = (&xid($clLine), &xid($svLine));
		if ($clId == $svId) {
			&join($clLine, $svLine);
			undef $clLine;
			undef $svLine;
		}
		elsif ($clId < $svId) {
			&join($clLine, undef()) if $JoinCount;
			undef $clLine;
		} 
		elsif ($clId < $svId) {
			undef $clLine;
		} 
		elsif ($svId == -1) {
			undef $clLine;
		} else {
			die("cl: $clLine\nsv: $svLine\nstopped");
		}
	}
}

sub join {
	my ($c, $s) = @_;

	# client line goes with no modifications
	chomp($c);
	print("$c\t ");

	# if server line exists, adjust to client time
	my @s;
	if ($s) {
		@s = split(/\s+/, $s);
		my $clStart = (split(/\s+/, $c))[2];
		my $svStart = $s[2];

		# print("\nclSt: $clStart svSt: $svStart\n");

		# remove xid, remember pid and iterId
		shift @s;
		my $pid = shift @s;
		my ($iter) = splice(@s, 1, 1);

		# convert deltas
		my $dstart = 
			($svHdr->{start} - $clHdr->{start})*1e6 +
			($svStart - $clStart);
		# print("dstart: $dstart\n");
		@s = map { $_ + $dstart } @s;

		# return pid and iterId
		splice(@s, 1, 0, $iter);
		unshift(@s, $pid);
	} else {
		# fill with -1s
		@s = (-1) x (1 + 1+1 + 4);
	}

	# print
	print(join("\t ", map { int($_); } @s));
	print("\n");
	
	$JoinCount++;
}

sub xid {
	my $line = shift;

	return $1 if $line =~ /^\s*(\-?\d+)/;
	die("wrong line format in '$line', stopped");
}

sub openLog {
	my $fname = shift;

	my $fh = new FileHandle("<$fname");
	die("cannot read $fname, stopped") unless $fh;
}


sub readHdr {
	my ($fh, $hdr) = @_;

	my $line;
	while (defined($line = $fh->getline())) {
		if ($line =~ /^\#\s+start: (\d+\.\d+)/) {
			$hdr->{start} = $1;
			$fh->getline(); # skip table header
			return $hdr;
		}
	}
	die("wrong header format, stopped");
}
